#include "string.h"
#include "stdlib.h"

#include "SDL.h"
#include "SDL_ttf.h"
#include "SDL_mixer.h"

#include "BList.h"
#include "auxiliar.h"

#include "variables.h"
#include "mog2.h"

#include "samples.h"
#include "animscript.h"

#include "BWindow.h"
#include "BActiveWindow.h"
#include "BIntegerEdDlg.h"
#include "BChoiceDlg.h"
#include "BVChoiceDlg.h"
#include "TileViewWnd.h"
#include "TileSelectionWnd.h"
#include "AnimScriptWnd.h"
#include "SampleSelectorDlg.h"

#include "interfaces.h"

#include "editor.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif

extern int SCREEN_X,SCREEN_Y;


int anim_cmd_id[ANIM_MAX_CMD] = {ANIM_CMD_NONE,
								 ANIM_CMD_SINGLE,
								 ANIM_CMD_SEQ,
								 ANIM_CMD_LOOP,
								 ANIM_CMD_GSEQ,
								 ANIM_CMD_GLOOP};

/*

  Tokens:
  %I: an integer
  %F: a frame
  %FL: a framelist
  %L: a list of all the anim commands

*/

char *anim_cmd_pat[ANIM_MAX_CMD] ={"NONE",
								   "SINGLE( %I , %F )",
								   "SEQ( %I , %FL )",
								   "LOOP( %I , %FL )",
								   "GSEQ( %L )",
								   "GLOOP( %L )"};


BViewAnimScriptWnd::BViewAnimScriptWnd(int x,int y,int dx,int dy,AnimSCRIPT *s,OBJECT *o,class EDITOR *e,TTF_Font *font) : BWindow(x,y,dx,dy,font)
{
	editor=e;
	script=s;
	object=o;
} /* BViewAnimScriptWnd::BViewAnimScriptWnd */ 


BViewAnimScriptWnd::~BViewAnimScriptWnd()
{
} /* BViewAnimScriptWnd::~BViewAnimScriptWnd */ 


bool BViewAnimScriptWnd::cycle(int stx,int sty)
{
	BWindow::cycle(stx,sty);

	if (local_mouse_x>old_stx &&
		local_mouse_y>old_sty &&
		local_mouse_x<old_endx &&
		local_mouse_y<old_endy &&
		mouse_button&SDL_BUTTON(1) &&
		!(old_mouse_button&SDL_BUTTON(1))) {
		click_on_script(script,4,4);
		dirty=true;
	} /* if */ 


	if (local_mouse_x>old_stx &&
		local_mouse_y>old_sty &&
		local_mouse_x<old_endx &&
		local_mouse_y<old_endy &&
		mouse_button&SDL_BUTTON(3) &&
		!(old_mouse_button&SDL_BUTTON(3))) {
		bool erase=false;
		right_click_on_script(script,4,4,&erase);
		dirty=true;

		if (erase) {
			delete script;
			script=new AnimSCRIPT();
		} /* if */ 
	} /* if */ 


	return true;
} /* BViewAnimScriptWnd::cycle */ 


void BViewAnimScriptWnd::contens_size(int *dx,int *dy)
{
	int max_x=0;
	int max_y=0;

	script_size(script,&max_x,&max_y);

	*dx=old_dx;
	*dy=old_dy;

	if (max_x+8>*dx) *dx=max_x+8;
	if (max_y+8>*dy) *dy=max_y+8;

} /* BViewAnimScriptWnd::contens_size */ 


void BViewAnimScriptWnd::draw_contens(SDL_Surface *sfc,int stx,int sty,int dx,int dy)
{

	SDL_FillRect(sfc,0,0);

	selected_script=0;

	draw_script(sfc,script,4-stx,4-sty);

	if (selected_script!=0) draw_rectangle(sfc,selected.x,selected.y,selected.w,selected.h,0xffffff);

	set_dirty();

} /* BViewAnimScriptWnd::draw_contens */ 


void BViewAnimScriptWnd::draw_script(SDL_Surface *sfc,AnimSCRIPT *script,int stx,int sty)
{
	int dx,dy;
	SDL_Rect r;
	char tmp[256];

	script_size(script,&dx,&dy);
	r.x=stx;
	r.y=sty;
	r.w=dx+4;
	r.h=dy+4;

	if (local_mouse_x>=stx+old_stx && local_mouse_y>=sty+old_sty &&
		local_mouse_x<stx+dx+4+old_stx && local_mouse_y<sty+dy+4+old_sty) {
		selected.x=stx;
		selected.y=sty;
		selected.w=dx+4;
		selected.h=dy+4;
		selected_script=script;
	} /* if */ 
	SDL_FillRect(sfc,&r,0x0000F0);

	r.x++;
	r.y++;
	r.w-=2;
	r.h-=2;
	SDL_FillRect(sfc,&r,0x0000A0);

	switch(script->command) {
	case ANIM_CMD_NONE:
		print_left("NONE",font,255,255,255,sfc,stx+2,sty+2);
		break;

	case ANIM_CMD_SINGLE:
		{
			int x;
			int tmp_dx,tmp_dy,y;

			x=stx+2;
			print_left("SINGLE(",font,255,255,255,sfc,x,sty+2);
			x+=get_text_width("SINGLE(",font)+TTF_FontHeight(font);

			sprintf(tmp,"%.3i",script->ipar);
			r.x=x;
			r.y=sty+2;
			r.w=get_text_width(tmp,font)+4;
			r.h=TTF_FontHeight(font)+4;
			SDL_FillRect(sfc,&r,0xA00000);
			if (local_mouse_x>=r.x+old_stx && local_mouse_y>=r.y+old_sty &&
				local_mouse_x<r.x+get_text_width(tmp,font)+4+old_stx && local_mouse_y<r.y+TTF_FontHeight(font)+4+old_sty) {
				selected.x=r.x;
				selected.y=r.y;
				selected.w=r.w;
				selected.h=r.h;
			} /* if */ 

			print_left(tmp,font,255,255,255,sfc,x+2,sty+4);
			x+=get_text_width(tmp,font)+4+TTF_FontHeight(font);

			print_left(",",font,255,255,255,sfc,x,sty+2);
			x+=get_text_width(",",font)+TTF_FontHeight(font);

			frame_size(&script->fpar,&tmp_dx,&tmp_dy);

			r.x=x;
			r.y=sty+2;
			r.w=tmp_dx;
			r.h=tmp_dy;
			SDL_FillRect(sfc,&r,0x00A000);
			if (local_mouse_x>=r.x+old_stx && local_mouse_y>=r.y+old_sty &&
				local_mouse_x<r.x+tmp_dx+old_stx && local_mouse_y<r.y+tmp_dy+old_sty) {
				selected.x=r.x;
				selected.y=r.y;
				selected.w=r.w;
				selected.h=r.h;
			} /* if */ 

			sprintf(tmp,"%.3i",script->fpar.frame);
			print_left(tmp,font,255,255,255,sfc,x+2,sty+4);

			y=sty+4+TTF_FontHeight(font)+2;
			if (script->fpar.p_offset_p!=0) {
				sprintf(tmp,"P: %.3i",script->fpar.p_offset_p);
				print_left(tmp,font,255,255,255,sfc,x+2,y);
				y+=TTF_FontHeight(font)+2;
			} /* if */ 
			if (script->fpar.p_offset_o!=0) {
				sprintf(tmp,"O: %.3i",script->fpar.p_offset_o);
				print_left(tmp,font,255,255,255,sfc,x+2,y);
				y+=TTF_FontHeight(font)+2;
			} /* if */ 
			if (script->fpar.sample!=0) {
				sprintf(tmp,"S: %s (%i)",script->fpar.sample->name,script->fpar.sample_volume);
				print_left(tmp,font,255,255,255,sfc,x+2,y);
				y+=TTF_FontHeight(font)+2;
			} /* if */ 

			x+=tmp_dx+TTF_FontHeight(font);
			print_left(")",font,255,255,255,sfc,x,sty+2);
		}
		break;

	case ANIM_CMD_SEQ:
	case ANIM_CMD_LOOP:

		{
			List<AnimFRAME> l;
			AnimFRAME *f;
			int x,y;
			int tmp_dx,tmp_dy;

			x=stx+2;
			if (script->command==ANIM_CMD_SEQ) {
				print_left("SEQ(",font,255,255,255,sfc,x,sty+2);
				x+=get_text_width("SEQ(",font)+TTF_FontHeight(font);
			} else {
				print_left("LOOP(",font,255,255,255,sfc,x,sty+2);
				x+=get_text_width("LOOP(",font)+TTF_FontHeight(font);
			} /* if */ 

			sprintf(tmp,"%.3i",script->ipar);
			r.x=x;
			r.y=sty+2;
			r.w=get_text_width(tmp,font)+4;
			r.h=TTF_FontHeight(font)+4;
			SDL_FillRect(sfc,&r,0xA00000);
			if (local_mouse_x>=r.x+old_stx && local_mouse_y>=r.y+old_sty &&
				local_mouse_x<r.x+get_text_width(tmp,font)+4+old_stx && local_mouse_y<r.y+TTF_FontHeight(font)+4+old_sty) {
				selected.x=r.x;
				selected.y=r.y;
				selected.w=r.w;
				selected.h=r.h;
			} /* if */ 

			print_left(tmp,font,255,255,255,sfc,x+2,sty+4);
			x+=get_text_width(tmp,font)+4+TTF_FontHeight(font);

			print_left(", [",font,255,255,255,sfc,x,sty+2);
			x+=get_text_width(", [",font)+TTF_FontHeight(font);

			l.Instance(script->fparlist);
			l.Rewind();
			while(l.Iterate(f)) {
				frame_size(f,&tmp_dx,&tmp_dy);
				r.x=x;
				r.y=sty+2;
				r.w=tmp_dx;
				r.h=tmp_dy;
				SDL_FillRect(sfc,&r,0x00A000);
				if (local_mouse_x>=r.x+old_stx && local_mouse_y>=r.y+old_sty &&
					local_mouse_x<r.x+tmp_dx+old_stx && local_mouse_y<r.y+tmp_dy+old_sty) {
					selected.x=r.x;
					selected.y=r.y;
					selected.w=r.w;
					selected.h=r.h;
				} /* if */ 

				sprintf(tmp,"%.3i",f->frame);
				print_left(tmp,font,255,255,255,sfc,x+2,sty+4);

				y=sty+4+TTF_FontHeight(font)+2;
				if (f->p_offset_p!=0) {
					sprintf(tmp,"P: %.3i",f->p_offset_p);
					print_left(tmp,font,255,255,255,sfc,x+2,y);
					y+=TTF_FontHeight(font)+2;
				} /* if */ 
				if (f->p_offset_o!=0) {
					sprintf(tmp,"O: %.3i",f->p_offset_o);
					print_left(tmp,font,255,255,255,sfc,x+2,y);
					y+=TTF_FontHeight(font)+2;
				} /* if */ 
				if (f->sample!=0) {
					sprintf(tmp,"S: %s (%i)",f->sample->name,f->sample_volume);
					print_left(tmp,font,255,255,255,sfc,x+2,y);
					y+=TTF_FontHeight(font)+2;
				} /* if */ 

				x+=tmp_dx+TTF_FontHeight(font);

				print_left(",",font,255,255,255,sfc,x,sty+2);
				x+=get_text_width(",",font)+TTF_FontHeight(font);
			} /* while */ 
			r.x=x;
			r.y=sty+2;
			r.w=get_text_width("END",font)+4;
			r.h=TTF_FontHeight(font)+4;
			SDL_FillRect(sfc,&r,0x00A000);
			if (local_mouse_x>=r.x+old_stx && local_mouse_y>=r.y+old_sty &&
				local_mouse_x<r.x+get_text_width("END",font)+4+old_stx && local_mouse_y<r.y+TTF_FontHeight(font)+4+old_sty) {
				selected.x=r.x;
				selected.y=r.y;
				selected.w=r.w;
				selected.h=r.h;
			} /* if */ 

			print_left("END",font,255,255,255,sfc,x+2,sty+4);
			x+=get_text_width("END",font)+TTF_FontHeight(font);

			print_left(")",font,255,255,255,sfc,x,sty+2);
		}
		break;
	case ANIM_CMD_GSEQ:
	case ANIM_CMD_GLOOP:
		{
			List<AnimSCRIPT> l;
			AnimSCRIPT *as;
			int x,y;

			if (script->command==ANIM_CMD_GSEQ) {
				print_left("GSEQ(",font,255,255,255,sfc,stx+2,sty+2);
				x=stx+2+get_text_width("GSEQ(",font)+TTF_FontHeight(font);
			} else {
				print_left("GLOOP(",font,255,255,255,sfc,stx+2,sty+2);
				x=stx+2+get_text_width("GLOOP(",font)+TTF_FontHeight(font);
			} /* if */ 
			y=sty+2;
			l.Instance(script->aparlist);
			l.Rewind();
			while(l.Iterate(as)) {
				int dx2,dy2;

				script_size(as,&dx2,&dy2);
				draw_script(sfc,as,x,y);
				y+=dy2+6;
			} /* while */ 

			r.x=x;
			r.y=y;
			r.w=get_text_width("END",font)+4;
			r.h=TTF_FontHeight(font)+4;
			SDL_FillRect(sfc,&r,0x00A000);
			if (local_mouse_x>=r.x+old_stx && local_mouse_y>=r.y+old_sty &&
				local_mouse_x<r.x+get_text_width("END",font)+4+old_stx && local_mouse_y<r.y+TTF_FontHeight(font)+4+old_sty) {
				selected.x=r.x;
				selected.y=r.y;
				selected.w=r.w;
				selected.h=r.h;
			} /* if */ 

			print_left("END",font,255,255,255,sfc,x+2,y+2);
			x+=get_text_width("END",font)+TTF_FontHeight(font);
			print_left(")",font,255,255,255,sfc,x+2,y+2);
		}	
		break;
	} /* switch */ 
} /* BViewAnimScriptWnd::draw_script */ 



bool BViewAnimScriptWnd::click_on_script(AnimSCRIPT *script,int stx,int sty)
{
	bool processed=false;
	int dx,dy;
	char tmp[256];

	script_size(script,&dx,&dy);

	switch(script->command) {
	case ANIM_CMD_NONE:
		break;
	case ANIM_CMD_SINGLE:
		if (local_mouse_x>=stx && local_mouse_y>=sty &&
			local_mouse_x<stx+dx+4 && local_mouse_y<sty+dy+4) {
			int x;
			int tmp_dx,tmp_dy;

			sprintf(tmp,"%.3i",script->ipar);
			frame_size(&script->fpar,&tmp_dx,&tmp_dy);
			x=stx+2+get_text_width("SINGLE(",font)+TTF_FontHeight(font);

			/* Timer */ 
			if (local_mouse_x>=x && local_mouse_y>=sty+2 &&
				local_mouse_x<x+get_text_width(tmp,font)+4 && local_mouse_y<sty+2+TTF_FontHeight(font)+4) {
				BIntegerEdDlg *dlg;
				int y,x;

				new_window_coordinates(140,60,&x,&y);
				dlg=new BIntegerEdDlg(x,y,140,60,"Enter time step:",&script->ipar,3,font);
				dlg->Execute();
				delete dlg;

				processed=true;
			} /* if */ 

			/* Tile */ 
			frame_size(&script->fpar,&tmp_dx,&tmp_dy);
			x=stx+get_text_width("SINGLE(",font)+TTF_FontHeight(font)+
			  get_text_width(tmp,font)+4+TTF_FontHeight(font)+
			  get_text_width(",",font)+TTF_FontHeight(font);
			if (local_mouse_x>=x && local_mouse_y>=sty+2 &&
				local_mouse_x<x+tmp_dx && local_mouse_y<sty+2+tmp_dy) {
				BTileSelectionWnd *dlg;
				int y,x,res;

				new_window_coordinates(140,200,&x,&y);
				dlg=new BTileSelectionWnd(x,y,140,200,object,&res,font);
				dlg->Execute();
				delete dlg;

				if (res!=-2) script->fpar.frame=res;
				processed=true;
			} /* if */ 

		} /* if */ 
		break;

	case ANIM_CMD_SEQ:
	case ANIM_CMD_LOOP:
		if (local_mouse_x>=stx && local_mouse_y>=sty &&
			local_mouse_x<stx+dx+4 && local_mouse_y<sty+dy+4) {
			int tmp_dx,tmp_dy;
			List<AnimFRAME> l;
			AnimFRAME *f;
			int x;

			sprintf(tmp,"%.3i",script->ipar);
			frame_size(&script->fpar,&tmp_dx,&tmp_dy);
			if (script->command==ANIM_CMD_SEQ) {
				x=stx+2+get_text_width("SEQ(",font)+TTF_FontHeight(font);
			} else {
				x=stx+2+get_text_width("LOOP(",font)+TTF_FontHeight(font);
			} /* if */ 

			/* Timer */ 
			if (local_mouse_x>=x && local_mouse_y>=sty+2 &&
				local_mouse_x<x+get_text_width(tmp,font)+4 && local_mouse_y<sty+2+TTF_FontHeight(font)+4) {
				BIntegerEdDlg *dlg;
				int y,x;

				new_window_coordinates(140,60,&x,&y);
				dlg=new BIntegerEdDlg(x,y,140,60,"Enter time step:",&script->ipar,3,font);
				dlg->Execute();
				delete dlg;

				processed=true;
			} /* if */ 

			x+=get_text_width(tmp,font)+4+TTF_FontHeight(font)+
			  get_text_width(", [",font)+TTF_FontHeight(font);

			l.Instance(script->fparlist);
			l.Rewind();
			while(l.Iterate(f)) {
				frame_size(f,&tmp_dx,&tmp_dy);
				/* Tile */ 
				if (local_mouse_x>=x && local_mouse_y>=sty+2 &&
					local_mouse_x<x+tmp_dx && local_mouse_y<sty+2+tmp_dy) {
					BTileSelectionWnd *dlg;
					int y,x,res;

					new_window_coordinates(140,200,&x,&y);
					dlg=new BTileSelectionWnd(x,y,140,200,object,&res,font);
					dlg->Execute();
					delete dlg;

					if (res!=-2) f->frame=res;
					processed=true;
				} /* if */ 

				x+=tmp_dx+TTF_FontHeight(font)+get_text_width(",",font)+TTF_FontHeight(font);
			} /* while */ 
			
			/* END: */ 
			if (local_mouse_x>=x && local_mouse_y>=sty+2 &&
				local_mouse_x<x+get_text_width("END",font)+4 && local_mouse_y<sty+2+TTF_FontHeight(font)+4) {
				AnimFRAME *f=new AnimFRAME();
				f->frame=0;
				f->p_offset_o=0;
				f->p_offset_p=0;
				f->sample=0;
				f->sample_volume=100;
				script->fparlist.Add(f);
				processed=true;
			} /* if */ 
		} /* if */ 
		break;

	case ANIM_CMD_GSEQ:
	case ANIM_CMD_GLOOP:
		if (local_mouse_x>=stx && local_mouse_y>=sty &&
			local_mouse_x<stx+dx+4 && local_mouse_y<sty+dy+4) {

			List<AnimSCRIPT> l;
			AnimSCRIPT *as;
			int x,y;

			if (script->command==ANIM_CMD_GSEQ) {
				x=stx+2+get_text_width("GSEQ(",font)+TTF_FontHeight(font);
			} else {
				x=stx+2+get_text_width("GLOOP(",font)+TTF_FontHeight(font);
			} /* if */ 

			y=sty+2;
			l.Instance(script->aparlist);
			l.Rewind();
			while(l.Iterate(as)) {
				int dx2,dy2;

				script_size(as,&dx2,&dy2);
				if (click_on_script(as,x,y)) processed=true;
				y+=dy2+6;
			} /* while */ 

			/* END: */ 
			if (local_mouse_x>=x && local_mouse_y>=y &&
				local_mouse_x<x+get_text_width("END",font)+4 && local_mouse_y<y+TTF_FontHeight(font)+4) {
				script->aparlist.Add(new AnimSCRIPT());
				processed=true;
			} /* if */ 

		} /* if */ 
		break;
	} /* switch */ 

	if (!processed &&
		local_mouse_x>=stx && local_mouse_y>=sty &&
		local_mouse_x<stx+dx+4 && local_mouse_y<sty+dy+4) {
		int res;
		BVChoiceDlg *dlg;

		dlg=new BVChoiceDlg(-1,-1,0,0,"Choose command:","none\nsingle\nseq\nloop\ngseq\ngloop\n",&res,font);
		dlg->Execute();
		delete dlg;

		switch(res) {
		case 1:script->command=ANIM_CMD_NONE;
			break;
		case 2:script->command=ANIM_CMD_SINGLE;
			  script->ipar=0;
			  script->fpar.frame=0;
			  script->fpar.p_offset_p=0;
			  script->fpar.p_offset_o=0;
			break;
		case 3:if (script->command!=ANIM_CMD_SEQ &&
				   script->command!=ANIM_CMD_LOOP) {
				   script->ipar=0;
				   script->fparlist.Delete();
			   } /* if */ 
			   script->command=ANIM_CMD_SEQ;		
			break;
		case 4:if (script->command!=ANIM_CMD_SEQ &&
				   script->command!=ANIM_CMD_LOOP) {
				   script->ipar=0;
				   script->fparlist.Delete();
			   } /* if */ 
			   script->command=ANIM_CMD_LOOP;			   
			break;
		case 5:if (script->command!=ANIM_CMD_GSEQ &&
				   script->command!=ANIM_CMD_GLOOP) {
				   script->aparlist.Delete();
			   } /* if */ 
			   script->command=ANIM_CMD_GSEQ;
			break;
		case 6:if (script->command!=ANIM_CMD_GSEQ &&
				   script->command!=ANIM_CMD_GLOOP) {
				   script->aparlist.Delete();
			   } /* if */ 
			   script->command=ANIM_CMD_GLOOP;			  
			break;
		} /* switch */ 
		processed=true;
	} /* if */ 

	return processed;
} /* BViewAnimScriptWnd::click_on_script */ 



bool BViewAnimScriptWnd::right_click_on_script(AnimSCRIPT *script,int stx,int sty,bool *erase)
{
	bool processed=false;
	int dx,dy;
	char tmp[256];

	script_size(script,&dx,&dy);

	switch(script->command) {
	case ANIM_CMD_NONE:
		break;
	case ANIM_CMD_SINGLE:
		if (local_mouse_x>=stx && local_mouse_y>=sty &&
			local_mouse_x<stx+dx+4 && local_mouse_y<sty+dy+4) {
			int x;
			int tmp_dx,tmp_dy;

			sprintf(tmp,"%.3i",script->ipar);
			frame_size(&script->fpar,&tmp_dx,&tmp_dy);
			x=stx+2+get_text_width("SINGLE(",font)+TTF_FontHeight(font);

			/* Timer */ 
			if (local_mouse_x>=x && local_mouse_y>=sty+2 &&
				local_mouse_x<x+get_text_width(tmp,font)+4 && local_mouse_y<sty+2+TTF_FontHeight(font)+4) {
				processed=true;
			} /* if */ 

			/* Tile */ 
			frame_size(&script->fpar,&tmp_dx,&tmp_dy);
			x=stx+get_text_width("SINGLE(",font)+TTF_FontHeight(font)+
			  get_text_width(tmp,font)+4+TTF_FontHeight(font)+
			  get_text_width(",",font)+TTF_FontHeight(font);
			if (local_mouse_x>=x && local_mouse_y>=sty+2 &&
				local_mouse_x<x+tmp_dx && local_mouse_y<sty+2+tmp_dy) {
				BVChoiceDlg *dlg;
				int res;

				dlg=new BVChoiceDlg(-1,-1,0,0,"Frame options?:","part offset\nobject offset\nsample\nno sample\nsample volume\n",&res,font);
				dlg->Execute();
				delete dlg;

				if (res==1) {
					BIntegerEdDlg *dlg;
					int y,x;

					new_window_coordinates(180,60,&x,&y);
					dlg=new BIntegerEdDlg(x,y,180,60,"Part precedence offset:",&(script->fpar.p_offset_p),6,font);
					dlg->Execute();
					delete dlg;
				} /* if */ 

				if (res==2) {
					BIntegerEdDlg *dlg;
					int y,x;

					new_window_coordinates(180,60,&x,&y);
					dlg=new BIntegerEdDlg(x,y,180,60,"Part precedence offset:",&(script->fpar.p_offset_o),6,font);
					dlg->Execute();
					delete dlg;			
				} /* if */ 

				if (res==3) {
					/* Add a sample: */ 
					int x,y;
					SAMPLE *s=0;
					SampleSelectorDlg *dlg;

					new_window_coordinates(200,200,&x,&y);
					dlg=new SampleSelectorDlg(x,y,200,200,editor,&s,font);
					dlg->Execute();
					delete dlg;

					if (s!=0) script->fpar.sample=s;
				} /* if */ 

				if (res==4) {
					script->fpar.sample=0;
				} /* if */ 

				if (res==5) {
					int x,y;
					BIntegerEdDlg *dlg;

					new_window_coordinates(140,60,&x,&y);

					dlg=new BIntegerEdDlg(x,y,140,60,"Enter volume:",&script->fpar.sample_volume,3,font);
					dlg->Execute();
					delete dlg;
					if (script->fpar.sample_volume<0) script->fpar.sample_volume=0;
					if (script->fpar.sample_volume>100) script->fpar.sample_volume=100;
				} /* if */ 

				processed=true;
			} /* if */ 

		} /* if */ 
		break;

	case ANIM_CMD_SEQ:
	case ANIM_CMD_LOOP:
		if (local_mouse_x>=stx && local_mouse_y>=sty &&
			local_mouse_x<stx+dx+4 && local_mouse_y<sty+dy+4) {
			int tmp_dx,tmp_dy;
			List<AnimFRAME> l,todelete;
			AnimFRAME *f;
			int x;

			sprintf(tmp,"%.3i",script->ipar);
			if (script->command==ANIM_CMD_SEQ) {
				x=stx+2+get_text_width("SEQ(",font)+TTF_FontHeight(font);
			} else {
				x=stx+2+get_text_width("LOOP(",font)+TTF_FontHeight(font);
			} /* if */ 

			/* Timer */ 
			if (local_mouse_x>=x && local_mouse_y>=sty+2 &&
				local_mouse_x<x+get_text_width(tmp,font)+4 && local_mouse_y<sty+2+TTF_FontHeight(font)+4) {
				processed=true;
			} /* if */ 

			x+=get_text_width(tmp,font)+4+TTF_FontHeight(font)+
			  get_text_width(", [",font)+TTF_FontHeight(font);

			l.Instance(script->fparlist);
			l.Rewind();
			while(l.Iterate(f)) {
				frame_size(f,&tmp_dx,&tmp_dy);
				/* Tile */ 
				if (local_mouse_x>=x && local_mouse_y>=sty+2 &&
					local_mouse_x<x+tmp_dx && local_mouse_y<sty+2+tmp_dy) {
					BVChoiceDlg *dlg;
					int res;

					dlg=new BVChoiceDlg(-1,-1,0,0,"Frame options:","part precedence\nobject precedence\ndelete\nsample\nno sample\nsample_volume\n",&res,font);
					dlg->Execute();
					delete dlg;

					if (res==1) {
						BIntegerEdDlg *dlg;
						int y,x;

						new_window_coordinates(160,60,&x,&y);
						dlg=new BIntegerEdDlg(x,y,160,60,"Part precedence offset:",&(f->p_offset_p),6,font);
						dlg->Execute();
						delete dlg;
					} /* if */ 

					if (res==2) {
						BIntegerEdDlg *dlg;
						int y,x;

						new_window_coordinates(160,60,&x,&y);
						dlg=new BIntegerEdDlg(x,y,140,60,"Part precedence offset:",&(f->p_offset_o),6,font);
						dlg->Execute();
						delete dlg;			
					} /* if */ 

					if (res==3) {
						todelete.Add(f);
					} /* if */ 

					if (res==4) {
						/* Add a sample: */ 
						int x,y;
						SAMPLE *s=0;
						SampleSelectorDlg *dlg;

						new_window_coordinates(200,200,&x,&y);
						dlg=new SampleSelectorDlg(x,y,200,200,editor,&s,font);
						dlg->Execute();
						delete dlg;

						if (s!=0) f->sample=s;
					} /* if */ 

					if (res==5) {
						f->sample=0;
					} /* if */ 

					if (res==6) {
						int x,y;
						BIntegerEdDlg *dlg;

						new_window_coordinates(140,60,&x,&y);

						dlg=new BIntegerEdDlg(x,y,140,60,"Enter volume:",&f->sample_volume,3,font);
						dlg->Execute();
						delete dlg;
						if (f->sample_volume<0) f->sample_volume=0;
						if (f->sample_volume>100) f->sample_volume=100;
					} /* if */ 

					processed=true;
				} /* if */ 

				x+=tmp_dx+TTF_FontHeight(font)+get_text_width(",",font)+TTF_FontHeight(font);
			} /* while */ 

			while(!todelete.EmptyP()) {
				f=todelete.ExtractIni();
				script->fparlist.DeleteElement(f);
				delete f;
			} /* while */ 
			
			/* END: */ 
			if (local_mouse_x>=x && local_mouse_y>=sty+2 &&
				local_mouse_x<x+get_text_width("END",font)+4 && local_mouse_y<sty+2+TTF_FontHeight(font)+4) {
				processed=true;
			} /* if */ 
		} /* if */ 
		break;

	case ANIM_CMD_GSEQ:
	case ANIM_CMD_GLOOP:
		if (local_mouse_x>=stx && local_mouse_y>=sty &&
			local_mouse_x<stx+dx+4 && local_mouse_y<sty+dy+4) {

			List<AnimSCRIPT> l,todelete;
			AnimSCRIPT *as;
			int x,y;

			if (script->command==ANIM_CMD_GSEQ) {
				x=stx+2+get_text_width("GSEQ(",font)+TTF_FontHeight(font);
			} else {
				x=stx+2+get_text_width("GLOOP(",font)+TTF_FontHeight(font);
			} /* if */ 

			y=sty+2;
			l.Instance(script->aparlist);
			l.Rewind();
			while(l.Iterate(as)) {
				int dx2,dy2;
				bool erase2=false;

				script_size(as,&dx2,&dy2);
				if (right_click_on_script(as,x,y,&erase2)) processed=true;
				if (erase2) todelete.Add(as);
				y+=dy2+6;
			} /* while */ 

			while(!todelete.EmptyP()) {
				as=todelete.ExtractIni();
				script->aparlist.DeleteElement(as);
				delete as;
			} /* while */ 

			/* END: */ 
			if (local_mouse_x>=x && local_mouse_y>=y &&
				local_mouse_x<x+get_text_width("END",font)+4 && local_mouse_y<y+TTF_FontHeight(font)+4) {
				processed=true;
			} /* if */ 

		} /* if */ 
		break;
	} /* switch */ 

	if (!processed &&
		local_mouse_x>=stx && local_mouse_y>=sty &&
		local_mouse_x<stx+dx+4 && local_mouse_y<sty+dy+4) {
		int res;
		BVChoiceDlg *dlg;

		dlg=new BVChoiceDlg(-1,-1,0,0,"Delete command?:","Delete\nCopy\nCut\nPaste\n",&res,font);
		dlg->Execute();
		delete dlg;

		if (res==1 || res==3) *erase=true;

		if (res==2 || res==3) {
			/* Copy */ 
			editor->se_clipboard_content=1;
			if (editor->se_clipboard_anim!=0) delete editor->se_clipboard_anim;
			editor->se_clipboard_anim=new AnimSCRIPT(*script);
		} /* if */ 

		if (res==4) {
			/* Paste */ 
			if (editor->se_clipboard_content==1) {
				script->copy_state(editor->se_clipboard_anim);
			} else {
				BChoiceDlg *dlg2=new BChoiceDlg(40,40,200,50,"The clipboard doesn't contain an animation","Accept",&res,editor->font);
				dlg2->Execute();
				delete dlg2;
			} /* if */ 
		} /* if */ 

		processed=true;
	} /* if */ 

	return processed;
} /* BViewAnimScriptWnd::right_click_on_script */ 



void BViewAnimScriptWnd::script_size(AnimSCRIPT *script,int *dx,int *dy)
{
	char tmp[256];

	switch(script->command) {
	case ANIM_CMD_NONE:
		*dx=get_text_width("NONE",font);
		*dy=TTF_FontHeight(font);
		break;
	case ANIM_CMD_SINGLE:
		{
			int x;
			int tmp_dx,tmp_dy;

			sprintf(tmp,"%.3i",script->ipar);
			frame_size(&script->fpar,&tmp_dx,&tmp_dy);
			x=get_text_width("SINGLE(",font)+TTF_FontHeight(font)+
			  get_text_width(tmp,font)+4+TTF_FontHeight(font)+
			  get_text_width(",",font)+TTF_FontHeight(font)+
			  tmp_dx+TTF_FontHeight(font)+
			  get_text_width(")",font);

			*dx=x;
			*dy=tmp_dy;
		}
		break;

	case ANIM_CMD_SEQ:
	case ANIM_CMD_LOOP:
		{
			List<AnimFRAME> l;
			AnimFRAME *f;
			int x;
			int tmp_dx,tmp_dy;

			sprintf(tmp,"%.3i",script->ipar);
			if (script->command==ANIM_CMD_SEQ) {
				x=get_text_width("SEQ(",font)+TTF_FontHeight(font);
			} else {
				x=get_text_width("LOOP(",font)+TTF_FontHeight(font);
			} /* if */ 
			x+=get_text_width(tmp,font)+4+TTF_FontHeight(font)+
			   get_text_width(", [",font)+TTF_FontHeight(font);

			*dy=TTF_FontHeight(font)+4;
			l.Instance(script->fparlist);
			l.Rewind();
			while(l.Iterate(f)) {
				frame_size(f,&tmp_dx,&tmp_dy);
				x+=tmp_dx+TTF_FontHeight(font)+get_text_width(",",font)+TTF_FontHeight(font);
				if (tmp_dy>*dy) *dy=tmp_dy;
			} /* while */ 
			x+=get_text_width("END",font)+TTF_FontHeight(font)+
			   get_text_width(")",font);

			*dx=x;
		}
		break;

	case ANIM_CMD_GSEQ:
	case ANIM_CMD_GLOOP:
		{
			List<AnimSCRIPT> l;
			AnimSCRIPT *as;
			int x,y;

			if (script->command==ANIM_CMD_GSEQ) {
				x=get_text_width("GSEQ(",font)+TTF_FontHeight(font);
			} else {
				x=get_text_width("GLOOP(",font)+TTF_FontHeight(font);
			} /* if */ 
			y=0;

			*dx=0;
			l.Instance(script->aparlist);
			l.Rewind();
			while(l.Iterate(as)) {
				int dx2,dy2;

				script_size(as,&dx2,&dy2);
				if (x+dx2+4>*dx) *dx=x+dx2+4;
				y+=dy2+6;
			} /* while */ 

			x+=get_text_width("END",font)+TTF_FontHeight(font);

			if (x+get_text_width(")",font)>*dx) *dx=x+get_text_width(")",font);
			*dy=y+TTF_FontHeight(font)+4;
		}
		break;

	default:
		*dx=0;
		*dy=0;
	} /* switch */ 
} /* BViewAnimScriptWnd::script_size */ 


void BViewAnimScriptWnd::frame_size(AnimFRAME *f,int *dx,int *dy)
{
	char tmp[80];
	
	sprintf(tmp,"%.3i",f->frame);
	*dx=get_text_width(tmp,font)+4;
	*dy=TTF_FontHeight(font)+4;
	if (f->p_offset_p!=0) {
		sprintf(tmp,"P: %.3i",f->p_offset_p);
		if (*dx<get_text_width(tmp,font)+4) *dx=get_text_width(tmp,font)+4;
		*dy+=TTF_FontHeight(font)+2;
	} /* if */ 
	if (f->p_offset_o!=0) {
		sprintf(tmp,"O: %.3i",f->p_offset_o);
		if (*dx<get_text_width(tmp,font)+4) *dx=get_text_width(tmp,font)+4;
		*dy+=TTF_FontHeight(font)+2;
	} /* if */ 
	if (f->sample!=0) {
		sprintf(tmp,"S: %s (%i)",f->sample->name,f->sample_volume);
		if (*dx<get_text_width(tmp,font)+4) *dx=get_text_width(tmp,font)+4;
		*dy+=TTF_FontHeight(font)+2;
	} /* if */ 
} /* BViewAnimScriptWnd::frame_size */ 
